@@ -1,10 +1,23 @@
+import builtins
 from copy import deepcopy
 from numbers import Number
-from types import ModuleType
-from typing import Any, Callable, Iterable, Iterator, overload, Union, List, Tuple, Dict
+from types import FunctionType, ModuleType
+from typing import (
+    Any,
+    Callable,
+    Iterable,
+    Iterator,
+    Sequence,
+    overload,
+    Union,
+    List,
+    Tuple,
+    Dict,
+)
 import inspect
 
 from macrokit.head import EXEC, Head
+from macrokit.type_map import _TYPE_MAP, register_type
 from macrokit._validator import validator
 from macrokit._symbol import Symbol
 
@@ -45,11 +58,23 @@ def _comma(a, b):
     return f"{a}, {b}".rstrip(", ")
 
 
+def _tuple_str(x: "Expr", i: int):
+    args = x.args
+    if len(args) == 1:
+        return f"{_s_(i)}({str_(args[0])},)"
+    else:
+        tup = ", ".join(str_(arg) for arg in args)
+        return f"{_s_(i)}({tup})"
+
+
 _STR_MAP: Dict[Head, Callable[["Expr", int], str]] = {
     Head.empty: lambda e, i: "",
     Head.getattr: lambda e, i: f"{str_(e.args[0], i)}.{str_(e.args[1])}",
     Head.getitem: lambda e, i: f"{str_(e.args[0], i)}[{str_(e.args[1])}]",
     Head.del_: lambda e, i: f"{_s_(i)}del {str_(e.args[0])}",
+    Head.tuple: _tuple_str,
+    Head.list: lambda e, i: f"{_s_(i)}[{', '.join(str_(arg) for arg in e.args)}]",
+    Head.braces: lambda e, i: f"{_s_(i)}{{{', '.join(str_(arg) for arg in e.args)}}}",
     Head.call: lambda e, i: f"{str_(e.args[0], i)}({sjoin(', ', e.args[1:])})",
     Head.assign: lambda e, i: f"{str_(e.args[0], i)} = {e.args[1]}",
     Head.kw: lambda e, i: f"{str_(e.args[0])}={str_(e.args[1])}",
@@ -179,15 +204,16 @@ class Expr:
         }
 
         # use registered modules
-        if Symbol._stored_symbols:
+        if _STORED_VALUES:
             format_dict: Dict[Symbol, _Expr] = {}
-            for id_, sym in Symbol._stored_symbols.items():
-                mod = Symbol._stored_variable_map[sym.name]
-                vstr = f"var{hex(id_)}"
+            for id_, (sym, obj) in _STORED_VALUES.items():
+                if isinstance(sym, Expr):
+                    continue
+                vstr = Symbol.symbol_str_for_id(id_)
                 format_dict[sym] = Symbol(vstr)
-                _glb[vstr] = mod
+                _glb[vstr] = obj
             # Modules will not be registered as alias ("np" will be "numpy" in macro).
-            # To avoid name collision, it is safer to rename them to "var0x...".
+            # To avoid name collision, it is safer to rename them to "var...".
             self = self.format(format_dict)
 
         if self.head in EXEC:
@@ -552,6 +578,15 @@ def _tuple(*args) -> tuple:
     return args
 
 
+# Stored symbols and the actual values. These will be used even after parse()
+# method, to make parse(str(macro)) executable.
+_STORED_VALUES: Dict[int, Tuple[_Expr, Any]] = {}
+
+
+# Map to speed up type check
+_SUBCLASS_MAP: Dict[type, type] = {}
+
+
 def symbol(obj: Any, constant: bool = True) -> _Expr:
     """
     Make a proper Symbol or Expr instance from any objects.
@@ -578,78 +613,48 @@ def symbol(obj: Any, constant: bool = True) -> _Expr:
 
     obj_type = type(obj)
     obj_id = id(obj)
-    if not constant or obj_id in Symbol._variables:
-        seq: Union[_Expr, str] = Symbol.make_symbol_str(obj)
+    _stored = False
+    if constant and obj_id in _STORED_VALUES:
+        seq: Union[str, _Expr] = _STORED_VALUES[obj_id][0]
+        _stored = True
+    elif not constant or obj_id in Symbol._variables:
+        seq = Symbol.make_symbol_str(obj)
         constant = False
-    elif obj_id in Symbol._stored_symbols.keys():
-        seq = Symbol._stored_symbols[obj_id]
-    elif obj_type in Symbol._type_map:
-        seq = Symbol._type_map[obj_type](obj)
-    elif obj_type in Symbol._subclass_map:
-        parent_type = Symbol._subclass_map[obj_type]
-        seq = Symbol._type_map[parent_type](obj)
-    elif isinstance(obj, tuple):
-        if len(obj) == 1:
-            # length 1 tuple have to be written as (a,) instead of (a).
-            seq = f"({symbol(obj[0])},)"
-        else:
-            seq = "(" + ", ".join(str(symbol(a)) for a in obj) + ")"
-        if obj_type is not tuple:
-            seq = obj_type.__name__ + seq
-    elif isinstance(obj, list):
-        seq = "[" + ", ".join(str(symbol(a)) for a in obj) + "]"
-        if obj_type is not list:
-            seq = f"{obj_type.__name__}({seq})"
-    elif isinstance(obj, dict):
-        seq = "{" + ", ".join(f"{symbol(k)}: {symbol(v)}" for k, v in obj.items()) + "}"
-        if obj_type is not dict:
-            seq = f"{obj_type.__name__}({seq})"
-    elif isinstance(obj, set):
-        if len(obj) == 0:
-            if obj_type is set:
-                seq = "set()"
-            else:
-                seq = f"{obj_type.__name__}()"
-        else:
-            seq = "{" + ", ".join(str(symbol(a)) for a in obj) + "}"
-            if obj_type is not set:
-                seq = f"{obj_type.__name__}({seq})"
-    elif isinstance(obj, frozenset):
-        seq = ", ".join(str(symbol(a)) for a in obj)
-        if obj_type is frozenset:
-            seq = f"frozenset({{{seq}}})"
-        else:
-            seq = f"{obj_type.__name__}({{{seq}}})"
-    elif isinstance(obj, Number):  # int, float, bool, ...
+    elif obj_type in _TYPE_MAP:
+        seq = _TYPE_MAP[obj_type](obj)
+        _stored = True
+    elif obj_type in _SUBCLASS_MAP:
+        parent_type = _SUBCLASS_MAP[obj_type]
+        seq = _TYPE_MAP[parent_type](obj)
+        _stored = True
+    elif isinstance(obj, Number):  # numpy scalars
+        _TYPE_MAP[obj_type] = str
         seq = str(obj)
-    elif isinstance(obj, ModuleType):
-        # Register module to the default namespace of Symbol class. This function is
-        # called every time a module type object is converted to a Symbol because users
-        # always have to pass the module object to the global variables when calling
-        # eval function.
-        if obj_id in Symbol._stored_symbols.keys():
-            sym = Symbol._stored_symbols[obj_id]
-        else:
-            *main, seq = obj.__name__.split(".")
-            sym = Symbol(seq, obj_id)
-            sym.constant = True
-            if len(main) == 0:
-                # submodules should not be registered
-                Symbol._stored_symbols[obj_id] = sym
-                Symbol._stored_variable_map[seq] = obj
-        return sym
-    elif hasattr(obj, "__name__"):
+    elif callable(obj) and hasattr(obj, "__name__"):  # most custom callables
         seq = obj.__name__
     else:
-        for k, func in Symbol._type_map.items():
+        # search for subclasses
+        for k, func in _TYPE_MAP.items():
             if isinstance(obj, k):
                 seq = func(obj)
-                Symbol._subclass_map[obj_type] = k
+                _SUBCLASS_MAP[obj_type] = k
                 break
         else:
             seq = Symbol.make_symbol_str(obj)
             constant = False
 
+    if isinstance(obj, (ModuleType, FunctionType)) and not _stored:
+        # Register modules/functions to the default namespace because users
+        # always have to pass the object to the global variables when
+        # calling the eval() function.
+        *main, seq = obj.__name__.split(".")
+        sym = Symbol(seq, obj_id)
+        sym.constant = True
+        if len(main) == 0:
+            # submodules should not be registered
+            _STORED_VALUES[obj_id] = (sym, obj)
+        return sym
+
     if isinstance(seq, (Symbol, Expr)):
         # The output of register_type can be a Symbol or Expr
         return seq
@@ -659,7 +664,7 @@ def symbol(obj: Any, constant: bool = True) -> _Expr:
         return sym
 
 
-def store(obj: Any) -> None:
+def store(obj: Any) -> _Expr:
     """Store a variable in a global Symbol namespace."""
     sym = symbol(obj)
     if not isinstance(sym, Symbol):
@@ -668,9 +673,67 @@ def store(obj: Any) -> None:
     name = sym.name
     if not name.isidentifier():
         raise ValueError(f"{name} is not an identifier.")
-    if name in Symbol._stored_variable_map:
-        _var = Symbol._stored_variable_map[name]
-        raise ValueError(f"Variable identifier {name} collides with {_var!r}")
-    Symbol._stored_symbols[obj_id] = sym
-    Symbol._stored_variable_map[name] = obj
-    return None
+    _STORED_VALUES[obj_id] = (sym, obj)
+    return sym
+
+
+def store_sequence(obj: Sequence[Any]) -> Symbol:
+    """Store a sequence object and make its contents expressed as varXX[i]."""
+    if not isinstance(obj, Sequence):
+        raise TypeError(f"Expected tuple, got {type(obj)}.")
+    obj_sym = Symbol.asvar(obj)
+    for idx, each in enumerate(obj):
+        expr = Expr(Head.getitem, [obj_sym, idx])
+        _id = id(each)
+        _STORED_VALUES[_id] = (expr, each)
+    return obj_sym
+
+
+def object_stored_at(id: int) -> Any:
+    """Return the object stored at the given ID."""
+    return _STORED_VALUES[id][1]
+
+
+def symbol_stored_at(id: int) -> _Expr:
+    """Return the Symbol object stored at the given ID."""
+    return _STORED_VALUES[id][0]
+
+
+register_type(tuple, lambda e: Expr(Head.tuple, [symbol(a) for a in e]))
+register_type(list, lambda e: Expr(Head.list, [symbol(a) for a in e]))
+
+
+@register_type(frozenset)
+def _fronset_expr(e: frozenset):
+    if len(e) == 0:
+        return Expr(Head.call, [frozenset])
+    else:
+        return Expr(Head.call, [frozenset, set(e)])
+
+
+@register_type(set)
+def _set_expr(e: set):
+    if len(e) == 0:
+        return Expr(Head.call, [set])
+    else:
+        return Expr(Head.braces, [symbol(a) for a in e])
+
+
+@register_type(dict)
+def _dict_expr(e: dict):
+    return Expr(
+        Head.braces,
+        [Expr(Head.annotate, [symbol(k), symbol(v)]) for k, v in e.items()],
+    )
+
+
+register_type(memoryview, lambda e: Expr(Head.call, [memoryview, e.tobytes()]))
+register_type(bytearray, lambda e: Expr(Head.call, [bytearray, bytes(e)]))
+register_type(slice, lambda e: Expr(Head.call, [slice, e.start, e.stop, e.step]))
+register_type(range, lambda e: Expr(Head.call, [range, e.start, e.stop, e.step]))
+
+# install builtin variables
+for name, obj in builtins.__dict__.items():
+    if name.startswith("_") or isinstance(obj, ModuleType):
+        continue
+    _STORED_VALUES[id(obj)] = (symbol(obj), obj)
