@@ -11,6 +11,9 @@ class Head(Enum):
     call = "call"
     assign = "assign"
     kw = "kw"
+    tuple = "tuple"
+    list = "list"
+    braces = "braces"
     comment = "comment"
     assert_ = "assert"
     unop = "unop"
